<?php
namespace Ep\Core;

use Throwable;
use WP_Post;

/**
 * Generalni Exception handler.
 *
 * Ako smo došli do ovog handlera, aplikacija je nepovratno uništena. Pokušavamo
 * na pristojan način objasniti korisniku što se desilo.
 *
 * Ideja je da redirektamo korisnika na stranicu s porukom. Ta stranica, ne bi i
 * sama smjela bacati nikakav Exception. Tu stranicu je moguće postaviti u EP Postavkama
 * u adminu, pod tabom 'Greške'. Koja god stranica je odabrana, response te stranice
 * će imati status kod 500.
 *
 * @package Ep\Core
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class ExceptionHandler
{
	/**
	 * Registrira ovaj handler.
	 */
	public function register()
	{
		ob_start();
		set_exception_handler([$this, 'handle']);

		add_filter('status_header', [$this, 'setStatusCode'], 99, 1);
	}

	/**
	 * Postavlja status kod responsa.
	 *
	 * Ako je id trenutnog posta isti kao i id odabrane stranice, zaglavlje responsa
	 * sadrži status kod 500.
	 *
	 * @param string $header Originalni header
	 *
	 * @return string
	 */
	public function setStatusCode($header)
	{
		$config = get_option('ep_errors', []);

		if (isset($config['error_page']) && !empty($config['error_page'])) {
			$pageId = absint($config['error_page']);

			global $post;

			if ($post instanceof WP_Post && $post->ID === $pageId) {
				return 'HTTP/1.1 500 Internal Server Error';
			}
		}

		return $header;
	}

	/**
	 * Hvata sve neuhvaćene Exceptione.
	 *
	 * Svaki neuhvaćeni exception ćemo negdje logirati i poslati administratoru
	 * na email.
	 *
	 * Nakon toga trebamo korisniku prikazati neku stranicu s porukom, tipa, došlo
	 * je do pogreške.
	 *
	 * @param Throwable $e
	 *
	 * @throws Throwable
	 */
	public function handle(Throwable $e): void
	{
		$this->logException($e);

		if ($adminEmail = get_option('admin_email')) {
			$this->email($e, $adminEmail);
		}

		$this->redirect();

		// Ako nismo redirektali, samo bacamo exception dalje.
		throw $e;
	}

	/**
	 * Sprema exception u log.
	 *
	 * @param Throwable $e
	 */
	protected function logException(Throwable $e)
	{
		// TODO: Napraviti logiranje.
	}

	/**
	 * Šalje exception na e-mail.
	 *
	 * @param Throwable $e     Exception koji šaljemo.
	 * @param string    $email E-mail adresa na koju šaljemo dati Exception.
	 */
	protected function email(Throwable $e, string $email)
	{
		$subject = sprintf('Greška na %s', $_SERVER['HTTP_HOST']);
		$message = <<< EOB
{$e->getMessage()}

{$e->getTraceAsString()}
EOB;

		wp_mail($email, $subject, $message, [
			sprintf('From: %s ep-webshop@%s', get_option('blogname'), $_SERVER['HTTP_HOST']),
			'Content-Type: text/plain; charset=utf-8',
		]);
	}

	/**
	 * Ako u postavkama imamo odabranu stranicu za generalnu grešku, redirektamo na nju.
	 */
	protected function redirect()
	{
		$config = get_option('ep_errors', []);

		if (!is_admin() && isset($config['error_page']) && !empty($config['error_page'])) {
			$pageId = absint($config['error_page']);

			global $post;

			if (!($post instanceof WP_Post && $post->ID === $pageId)) {
				$pageUrl = get_permalink($pageId);

				wp_redirect($pageUrl);
				exit;
			}
		}
	}
}