<?php
namespace Ep\Gath\Hydrator;

use Ep\Gath\Entity\Discount;
use Ep\Gath\Entity\Rebate;
use Interop\Container\ContainerInterface;
use Interop\Container\Exception\ContainerException;
use Zend\Hydrator\HydratorInterface;
use Zend\Hydrator\HydratorPluginManager;
use Zend\Hydrator\NamingStrategy\MapNamingStrategy;
use Zend\Hydrator\Reflection as ReflectionHydrator;
use Zend\Hydrator\Strategy\ClosureStrategy;
use Zend\ServiceManager\Exception\ServiceNotCreatedException;
use Zend\ServiceManager\Exception\ServiceNotFoundException;
use Zend\ServiceManager\Factory\FactoryInterface;

/**
 * Stvara novu instancu hidratora za Price objekt.
 *
 * @package Ep\Gath\Hydrator
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class PriceHydratorFactory implements FactoryInterface
{
	/**
	 * Stvara objekt
	 *
	 * @param  ContainerInterface $container
	 * @param  string $requestedName
	 * @param  null|array $options
	 *
	 * @return object
	 * @throws ServiceNotFoundException if unable to resolve the service.
	 * @throws ServiceNotCreatedException if an exception is raised when
	 *     creating a service.
	 * @throws ContainerException if any other error occurs
	 */
	public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
	{
		/** @var HydratorPluginManager $hydrators */
		$hydrators = $container->get('HydratorManager');
		$hydrator = new ReflectionHydrator();
		/** @var HydratorInterface $discountHydrator */
		$discountHydrator = $hydrators->get(Discount::class);
		/** @var HydratorInterface $rebateHydrator */
		$rebateHydrator = $hydrators->get(Rebate::class);

		$hydrator->setNamingStrategy(new MapNamingStrategy([
			'ProductCode' => 'sku',
			'TaxRate' => 'taxRate',
			'PriceRetail' => 'priceRetail',
			'PriceWholesale' => 'priceWholesale',
			'Discount' => 'discounts',
			'RabPolicy' => 'rebate',
		]));

		$hydrator->addStrategy('discounts', new ClosureStrategy(
			// extract
			function() {
				// za sada ne trebamo izvlačiti podatke iz entiteta
				return [];
			},
			// hydrate
			function($discounts) use ($discountHydrator) {
				$hydrated = [];

				foreach ($discounts as $discount) {
					$hydrated[] = $discountHydrator->hydrate($discount, new Discount());
				}

				return $hydrated;
			}
		));

		$hydrator->addStrategy('rebate', new ClosureStrategy(
			// extract
			function() {
				// za sada ne trebamo izvlačiti podatke iz entiteta
				return [];
			},
			// hydrate
			function($rebates) use ($rebateHydrator) {
				return is_array($rebates) && count($rebates) > 0
					? $rebateHydrator->hydrate($rebates[0], new Rebate())
					: null;
			}
		));

		return $hydrator;
	}
}