<?php
namespace Ep\Gath\WordPress;

use Ep\Core\UserProviderInterface;
use Ep\Gath\GathService;
use Ep\WordPress\Service\MetadataProviderInterface;
use WP_Post;

/**
 * Class AbstractMetadataProvider
 *
 * @package Ep\Gath\WordPress
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
abstract class ProductMetadataProvider implements MetadataProviderInterface
{
	/** @var string[] */
	protected $metaKeys = [];
	/** @var UserProviderInterface */
	protected $users;
	/** @var GathService */
	protected $gath;
	/** @var array */
	protected $preload = [];
	/** @var array */
	protected $loaded = [];

	/**
	 * MetadataProvider constructor.
	 *
	 * @param GathService $gath
	 * @param UserProviderInterface $users
	 */
	public function __construct(GathService $gath, UserProviderInterface $users)
	{
		$this->gath = $gath;
		$this->users = $users;

		add_action('wp', function() {
			//$this->load();
		});
	}

	/**
	 * Kada god izvučemo postove iz WordPressa, ako su proizvodi, njihove skuove dodajemo
	 * u listu koju ćemo kasnije učitati odjednom.
	 *
	 * @param array|WP_Post[] $posts
	 */
	public function preloadMetadata(array $posts): void
	{
		$posts = array_filter($posts);
		$config = get_option('ep_gath', []);
		$preloadGrouped = !isset($config['dontPreloadGrouped']) || '0' === $config['dontPreloadGrouped'];

		foreach ($posts as $post) {
			if (is_object($post)) {
				if ($post->post_type !== 'product') {
					continue;
				}

				$postId = $post->ID;
			} else {
				$postId = $post;
			}

			if ($postId > GathService::PRODUCT_ID_OFFSET) {
				$postId -= GathService::PRODUCT_ID_OFFSET;
			}

			if (array_key_exists($postId, $this->loaded)) {
				continue;
			}

			if ( !isset($this->preload[$postId]) && ($sku = get_post_meta($postId, '_sku', true))) {
				// Ako je onaj naš grupirani proizvod, dodajemo skuove sadržanih simplova u preload.
				// Pošto grupirani proizvodi u biti ne postoje, njih ćemo preskočiti.
				if (preg_match('/^v_art/', $sku)) {
					if (!$preloadGrouped) {
						continue;
					}

					$childrenIds = get_post_meta($postId, '_children', true);

					foreach ($childrenIds as $childId) {
						if (!array_key_exists($childId, $this->loaded) && ($childSku = get_post_meta($childId, '_sku', true))) {
							$this->preload[$childId] = $childSku;
						}
					}
				} else {
					$this->preload[$postId] = $sku;
				}
			}
		}

		asort($this->preload);
	}

	/**
	 * Učitava metapodatke za proizvode dodane u preloadMetadata().
	 *
	 * Svaka implementacija getMetadata metode, mora prvo pozvati ovu metodu.
	 */
	protected function load()
	{
		if (count($this->preload) > 0) {
			$this->loadMetadata();
			$this->preload = [];
		}
	}

	/**
	 * Učitava metapodatke za proizvode u $this->load.
	 */
	abstract protected function loadMetadata();

	/**
	 * Vraća true za meta ključeve koje ovaj provider nudi.
	 *
	 * @param string $metaKey
	 *
	 * @return bool
	 */
	public function hasMetadata(string $metaKey): bool
	{
		return defined('REST_REQUEST')
			? false
			: in_array($metaKey, $this->metaKeys);
	}
}