<?php
namespace Ep\WooCommerce;

use Ep\WordPress\Service\AbstractWordPressService;
use WP_Post;
use WP_Term;

/**
 * Class AutoMenu
 *
 * @package Ep\WooCommerce
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class AutoMenu extends AbstractWordPressService
{
	public function attach(): void
	{
		add_action('created_product_cat', [$this, 'addProductCat']);
		add_action('delete_product_cat', [$this, 'removeProductCat']);
		add_action('wp_update_nav_menu', [$this, 'update']);
	}

	/**
	 * Dodaje js na stranicu za uređivanje izbornika.
	 *
	 * Ovo nam omogućava da umetnemo polje u postavke izbornika.
	 */
	public function adminAssets(): void
	{
		if ('nav-menus' === get_current_screen()->base) {
			global $nav_menu_selected_id;
			$options = (array) get_option('nav_menu_options');

			$manifest = json_decode(file_get_contents(EP_WEBSHOP_DIR . '/dist/manifest.json'), true);

			wp_enqueue_script('ep-admin-auto-menu', sprintf('%s/dist/%s', EP_WEBSHOP_URL, $manifest['ep-admin-auto-menu.js']), ['jquery'], false, true);
			wp_localize_script('ep-admin-auto-menu', 'ep_wc_auto_menu', [
				'auto_add_product_cats' => in_array($nav_menu_selected_id, (array) @$options['auto_add_product_cats']),
				'auto_menu_children_only' => in_array($nav_menu_selected_id, (array) @$options['auto_menu_children_only']),
			]);
		}
	}

	/**
	 * Sprema auto-menu postavke.
	 *
	 * @param int $menuId
	 */
	public function update(int $menuId)
	{
		$options = (array) get_option('nav_menu_options');

		if (!isset($options['auto_add_product_cats'])) {
			$options['auto_add_product_cats'] = [];
		}

		$autoAdd = !empty($_POST['auto-add-product-cats']);

		if ($autoAdd) {
			$options['auto_add_product_cats'][] = $menuId;
			$options['auto_add_product_cats'] = array_unique($options['auto_add_product_cats']);
		} else {
			$index = array_search($menuId, $options['auto_add_product_cats']);

			if (isset($options['auto_add_product_cats'][$index])) {
				unset($options['auto_add_product_cats'][$index]);
			}
		}

		if (!isset($options['auto_menu_children_only'])) {
			$options['auto_menu_children_only'] = [];
		}

		$childrenOnly = !empty($_POST['auto-menu-children-only']);

		if ($childrenOnly) {
			$options['auto_menu_children_only'][] = $menuId;
			$options['auto_menu_children_only'] = array_unique($options['auto_menu_children_only']);
		} else {
			$index = array_search($menuId, $options['auto_menu_children_only']);

			if (isset($options['auto_menu_children_only'][$index])) {
				unset($options['auto_menu_children_only'][$index]);
			}
		}

		update_option('nav_menu_options', $options);
	}

	/**
	 * Dodaje sve nove kategorije proizvoda u odabrane izbornike.
	 *
	 * @param int $categoryId
	 */
	public function addProductCat(int $categoryId)
	{
		$options = (array) get_option('nav_menu_options');

		if (!isset($options['auto_add_product_cats']) || !is_array($options['auto_add_product_cats']) || empty($options['auto_add_product_cats'])) {
			return;
		}

		$category = get_term($categoryId);

		foreach ($options['auto_add_product_cats'] as $menuId) {
			$navItemArgs = [
				'menu-item-object' => 'product_cat',
				'menu-item-object-id' => $category->term_id,
				'menu-item-type' => 'taxonomy',
				'menu-item-status' => 'publish',
			];

			if ($category->parent > 0) {
				/** @var WP_Post[]|object[] $menuNavItems */
				$menuNavItems = wp_get_nav_menu_items($menuId) ?: [];

				foreach ($menuNavItems as $navItem) {
					if ('product_cat' === $navItem->object && $category->parent == $navItem->object_id) {
						$navItemArgs['menu-item-parent-id'] = $navItem->ID;
						break;
					}
				}
			}

			wp_update_nav_menu_item($menuId, 0, $navItemArgs);
		}
	}

	/**
	 * Nakon brisanja kategorije proizvoda uklanjamo ju iz odabranih izbornika.
	 *
	 * @param int $categoryId
	 */
	public function removeProductCat(int $categoryId)
	{
		$options = (array) get_option('nav_menu_options');

		if (!isset($options['auto_add_product_cats']) || !is_array($options['auto_add_product_cats'])) {
			return;
		}

		foreach ($options['auto_add_product_cats'] as $menuId) {
			/** @var WP_Post[]|object[] $menuNavItems */
			$menuNavItems = wp_get_nav_menu_items($menuId) ?: [];

			foreach ($menuNavItems as $navItem) {
				if ('product_cat' === $navItem->object && $categoryId == $navItem->object_id) {
					wp_delete_post($navItem->ID);
				}
			}
		}
	}
}