<?php
namespace Ep\WordPress;

use Ep\WordPress\Settings\SettingsProviderInterface;
use Zend\Form\ElementFactory;
use Zend\Stdlib\ArrayUtils;

/**
 * Konfiguracija WordPress komponenti
 *
 * @package Ep\WordPress
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class ConfigProvider implements SettingsProviderInterface
{
	public function __invoke()
	{
		$services = [
			'WordPressI18n',
			'WordPressMail',
			'WordPressMetadata',
			'WordPressUsers',
			'WordPressCleaningService',
			'FormcraftFileDownload',
		];

		$webshopConfig = $this->getDefaults()['ep_webshop'];

		if (function_exists('get_option')) {
			$devConfig = get_option('ep_dev', []);

			if (isset($devConfig['panel']) && '1' === $devConfig['panel']) {
				$services[] = 'WordPressDevPanel';
			}

			$webshopConfig = ArrayUtils::merge($webshopConfig, get_option('ep_webshop', []));
		}



		return [
			'dependencies' => $this->getDependencyConfig(),
			'form_elements' => $this->getFormElementConfig(),
			'webshop' => $webshopConfig,

			'wordpress' => [
				'services' => $services,

				'admin_services' => [
					'WordPressSettings',
				],

				'metadata_providers' => [],

				'settings_providers' => [],
			],
		];
	}

	/**
	 * Vraća konfiguraciju za ServiceManager
	 *
	 * @return array
	 */
	public function getDependencyConfig()
	{
		return [
			'aliases' => [
				'WordPress' => WordPressManager::class,
				'WordPressCleaningService' => Tools\CleaningService::class,
				'WordPressDevPanel' => Service\DevPanel::class,
				'WordPressI18n' => Service\I18nService::class,
				'WordPressMail' => Service\MailService::class,
				'WordPressMetadata' => Service\MetadataService::class,
				'WordPressSettings' => Settings\SettingsService::class,
				'WordPressUsers' => Service\UserService::class,
				'FormcraftFileDownload' => Formcraft\FileDownloadAddon::class,
			],
			'factories' => [
				WordPressManager::class         => WordPressManagerFactory::class,
				Service\MetadataService::class  => Service\MetadataServiceFactory::class,
				Settings\SettingsService::class => Settings\SettingsServiceFactory::class,
			],
		];
	}

	/**
	 * Vraća konfiguraciju FormElementManagera.
	 *
	 * @return array
	 */
	public function getFormElementConfig()
	{
		return [
			'aliases' => [
				'postselect' => Form\PostSelect::class,
				'postSelect' => Form\PostSelect::class,
				'PostSelect' => Form\PostSelect::class,
				'posttypeselect' => Form\PostTypeSelect::class,
				'postTypeSelect' => Form\PostTypeSelect::class,
				'PostTypeSelect' => Form\PostTypeSelect::class,
			],

			'factories' => [
				Form\PostSelect::class     => ElementFactory::class,
				Form\PostTypeSelect::class => ElementFactory::class,
			],
		];
	}

	/**
	 * Vraća zadane postavke koje provider pruža.
	 *
	 * @return array
	 */
	public function getDefaults()
	{
		return [
			'ep_webshop' => [
				'type' => 'retail',
				'enable_cart_import_csv' => '0',
				'login_for_prices' => '0',
				'login_for_prices_text' => '',
			],

			'ep_smtp' => [
				'enabled' => '0',
				'host' => '',
				'port' => 25,
				'username' => '',
				'password' => '',
			],

			'ep_errors' => [
				'error_page' => '',
				'error_block' => '',
			],

			'ep_dev' => [
				'panel' => '0',
			],
		];
	}

	/**
	 * Vraća forme za postavke.
	 *
	 * @return array
	 */
	public function getForms()
	{
		return [
			'ep_webshop' => 'settings-webshop',
			'ep_smtp' => 'settings-smtp',
			'ep_errors' => 'settings-errors',
			'ep_dev' => 'settings-dev',
		];
	}

	/**
	 * Vraća tabove u control panelu koje ovaj provider definira.
	 *
	 * @return array
	 */
	public function getTabs()
	{
		return [[
			'id' => 'general',
			'title' => 'Općenito',
			'priority' => 10,
			'optionKeys' => [
				'ep_webshop',
				'ep_smtp',
				'ep_errors',
			],
		], [
			'id' => 'dev',
			'title' => 'Razvoj',
			'priority' => 0,
			'optionKeys' => [
				'ep_dev',
			],
		]];
	}
}